@@ -8,40 +8,43 @@ mod owned;
 mod stack;
 
 use std::io::Write;
+use std::path::Path;
 
 pub struct Config<'a> {
     pub dry_run: bool,
-    pub force: bool,
+    pub force_author: bool,
+    pub force_detach: bool,
     pub base: Option<&'a str>,
     pub and_rebase: bool,
+    pub rebase_options: &'a Vec<&'a str>,
     pub whole_file: bool,
     pub one_fixup_per_commit: bool,
-    pub logger: &'a slog::Logger,
 }
 
-pub fn run(config: &mut Config) -> Result<()> {
+pub fn run(logger: &slog::Logger, config: &Config) -> Result<()> {
     let repo = git2::Repository::open_from_env()?;
-    debug!(config.logger, "repository found"; "path" => repo.path().to_str());
+    debug!(logger, "repository found"; "path" => repo.path().to_str());
 
-    // here, we default to the git config value,
-    // if the flag was not provided in the CLI.
-    //
-    // in the future, we'd likely want to differentiate between
-    // a "non-provided" option, vs an explicit --no-<option>
-    // that disables a behavior, much like git does.
-    // e.g. user may want to overwrite a config value with
-    // --no-one-fixup-per-commit -- then, defaulting to the config value
-    // like we do here is no longer sufficient. but until then, this is fine.
-    //
-    config.one_fixup_per_commit |= config::one_fixup_per_commit(&repo);
-
-    run_with_repo(config, &repo)
+    run_with_repo(&logger, &config, &repo)
 }
 
-fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
-    let stack = stack::working_stack(&repo, config.base, config.force, config.logger)?;
+fn run_with_repo(logger: &slog::Logger, config: &Config, repo: &git2::Repository) -> Result<()> {
+    if !config.rebase_options.is_empty() && !config.and_rebase {
+        return Err(anyhow!(
+            "REBASE_OPTIONS were specified without --and-rebase flag"
+        ));
+    }
+
+    let config = config::unify(&config, repo);
+    let stack = stack::working_stack(
+        repo,
+        config.base,
+        config.force_author,
+        config.force_detach,
+        logger,
+    )?;
     if stack.is_empty() {
-        crit!(config.logger, "No commits available to fix up, exiting");
+        crit!(logger, "No commits available to fix up, exiting");
         return Ok(());
     }
 
@@ -83,7 +86,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
                     diff_options.as_mut(),
                 )?,
             )?;
-            trace!(config.logger, "parsed commit diff";
+            trace!(logger, "parsed commit diff";
                    "commit" => commit.id().to_string(),
                    "diff" => format!("{:?}", diff),
             );
@@ -91,10 +94,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
         }
 
         let summary_counts = stack::summary_counts(&stack);
-        (
-            stack.into_iter().zip(diffs.into_iter()).collect(),
-            summary_counts,
-        )
+        (stack.into_iter().zip(diffs).collect(), summary_counts)
     };
 
     let mut head_tree = repo.head()?.peel_to_tree()?;
@@ -103,7 +103,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
         None,
         diff_options.as_mut(),
     )?)?;
-    trace!(config.logger, "parsed index";
+    trace!(logger, "parsed index";
            "index" => format!("{:?}", index),
     );
 
@@ -118,7 +118,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
     'patch: for index_patch in index.iter() {
         let old_path = index_patch.new_path.as_slice();
         if index_patch.status != git2::Delta::Modified {
-            debug!(config.logger, "skipped non-modified hunk";
+            debug!(logger, "skipped non-modified hunk";
                     "path" => String::from_utf8_lossy(old_path).into_owned(),
                     "status" => format!("{:?}", index_patch.status),
             );
@@ -130,7 +130,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
         let mut preceding_hunks_offset = 0isize;
         let mut applied_hunks_offset = 0isize;
         'hunk: for index_hunk in &index_patch.hunks {
-            debug!(config.logger, "next hunk";
+            debug!(logger, "next hunk";
                    "header" => index_hunk.header(),
                    "path" => String::from_utf8_lossy(old_path).into_owned(),
             );
@@ -175,7 +175,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
             // |   -5,1 +3,0    | -5,1 +4,0 |    -4,1 +3,0     |
             // |----------------|-----------|------------------|
 
-            debug!(config.logger, "";
+            debug!(logger, "";
                 "to apply" => hunk_to_apply.header(),
                 "to commute" => isolated_hunk.header(),
                 "preceding hunks" => format!("{}/{}", applied_hunks_offset, preceding_hunks_offset),
@@ -188,8 +188,8 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
             let mut commuted_old_path = old_path;
             let mut commuted_index_hunk = isolated_hunk;
 
-            'commit: for &(ref commit, ref diff) in &stack {
-                let c_logger = config.logger.new(o!(
+            'commit: for (commit, diff) in &stack {
+                let c_logger = logger.new(o!(
                     "commit" => commit.id().to_string(),
                 ));
                 let next_patch = match diff.by_new(commuted_old_path) {
@@ -252,7 +252,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
                 // so there is no commit to absorb it into
                 None => {
                     warn!(
-                        config.logger,
+                        logger,
                         "Could not find a commit to fix up, use \
                          --base to increase the search range."
                     );
@@ -271,7 +271,8 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
         }
     }
 
-    hunks_with_commit.sort_by_key(|h| h.dest_commit.id());
+    let target_always_sha: bool = config::fixup_target_always_sha(repo);
+
     // * apply all hunks that are going to be fixed up into `dest_commit`
     // * commit the fixup
     // * repeat for all `dest_commit`s
@@ -284,7 +285,7 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
         .zip(hunks_with_commit.iter().skip(1).map(Some).chain([None]))
     {
         let new_head_tree = apply_hunk_to_tree(
-            &repo,
+            repo,
             &head_tree,
             &current.hunk_to_apply,
             &current.index_patch.old_path,
@@ -301,11 +302,14 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
             // https://docs.rs/git2/0.7.5/src/git2/repo.rs.html#998
             // https://libgit2.org/libgit2/#HEAD/group/commit/git_commit_create
             let dest_commit_id = current.dest_commit.id().to_string();
-            let dest_commit_locator = current
-                .dest_commit
-                .summary()
-                .filter(|&msg| summary_counts[msg] == 1)
-                .unwrap_or(&dest_commit_id);
+            let dest_commit_locator = match target_always_sha {
+                true => &dest_commit_id,
+                false => current
+                    .dest_commit
+                    .summary()
+                    .filter(|&msg| summary_counts[msg] == 1)
+                    .unwrap_or(&dest_commit_id),
+            };
             let diff = repo
                 .diff_tree_to_tree(Some(&head_commit.tree()?), Some(&new_head_tree), None)?
                 .stats()?;
@@ -319,12 +323,12 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
                     &head_tree,
                     &[&head_commit],
                 )?)?;
-                info!(config.logger, "committed";
+                info!(logger, "committed";
                       "commit" => head_commit.id().to_string(),
                       "header" => format!("+{},-{}", diff.insertions(), diff.deletions()),
                 );
             } else {
-                info!(config.logger, "would have committed";
+                info!(logger, "would have committed";
                       "fixup" => dest_commit_locator,
                       "header" => format!("+{},-{}", diff.insertions(), diff.deletions()),
                 );
@@ -347,14 +351,14 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
     if patches_considered == 0 {
         if index_was_empty && !we_added_everything_to_index {
             warn!(
-                config.logger,
+                logger,
                 "No changes staged, try adding something \
                  to the index or set {} = true",
                 config::AUTO_STAGE_IF_NOTHING_STAGED_CONFIG_NAME
             );
         } else {
             warn!(
-                config.logger,
+                logger,
                 "Could not find a commit to fix up, use \
                  --base to increase the search range."
             )
@@ -368,7 +372,30 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
         assert!(number_of_parents <= 1);
 
         let mut command = Command::new("git");
-        command.args(&["rebase", "--interactive", "--autosquash", "--autostash"]);
+
+        // We'd generally expect to be run from within the repository, but just in case,
+        // try to have git run rebase from the repository root.
+        // This simplifies writing tests that execute from within git-absorb's source directory
+        // but operate on temporary repositories created elsewhere.
+        // (The tests could explicitly change directories, but then must be serialized.)
+        let repo_path = repo.path().parent().map(Path::to_str).flatten();
+        match repo_path {
+            Some(path) => {
+                command.args(["-C", path]);
+            }
+            _ => {
+                warn!(
+                    logger,
+                    "Could not determine repository path for rebase. Running in current directory."
+                );
+            }
+        }
+
+        command.args(["rebase", "--interactive", "--autosquash", "--autostash"]);
+
+        for arg in config.rebase_options {
+            command.arg(arg);
+        }
 
         if number_of_parents == 0 {
             command.arg("--root");
@@ -379,9 +406,14 @@ fn run_with_repo(config: &Config, repo: &git2::Repository) -> Result<()> {
             command.arg(&base_commit_sha);
         }
 
-        // Don't check that we have successfully absorbed everything, nor git's
-        // exit code -- as git will print helpful messages on its own.
-        command.status().expect("could not run git rebase");
+        if config.dry_run {
+            info!(logger, "would have run git rebase"; "command" => format!("{:?}", command));
+        } else {
+            debug!(logger, "running git rebase"; "command" => format!("{:?}", command));
+            // Don't check that we have successfully absorbed everything, nor git's
+            // exit code -- as git will print helpful messages on its own.
+            command.status().expect("could not run git rebase");
+        }
     }
 
     Ok(())
@@ -483,103 +515,20 @@ fn index_stats(repo: &git2::Repository) -> Result<git2::DiffStats> {
 
 #[cfg(test)]
 mod tests {
-    use std::path::{Path, PathBuf};
+    use git2::message_trailers_strs;
+    use std::path::PathBuf;
 
     use super::*;
-
-    struct Context {
-        repo: git2::Repository,
-        dir: tempfile::TempDir,
-    }
-
-    impl Context {
-        fn join(&self, p: &Path) -> PathBuf {
-            self.dir.path().join(p)
-        }
-    }
-
-    /// Prepare a fresh git repository with an initial commit and a file.
-    fn prepare_repo() -> (Context, PathBuf) {
-        let dir = tempfile::tempdir().unwrap();
-        let repo = git2::Repository::init(dir.path()).unwrap();
-
-        let path = PathBuf::from("test-file.txt");
-        std::fs::write(
-            dir.path().join(&path),
-            br#"
-line
-line
-
-more
-lines
-"#,
-        )
-        .unwrap();
-
-        // make the borrow-checker happy by introducing a new scope
-        {
-            let tree = add(&repo, &path);
-            let signature = repo
-                .signature()
-                .or_else(|_| git2::Signature::now("nobody", "nobody@example.com"))
-                .unwrap();
-            repo.commit(
-                Some("HEAD"),
-                &signature,
-                &signature,
-                "Initial commit.",
-                &tree,
-                &[],
-            )
-            .unwrap();
-        }
-
-        (Context { repo, dir }, path)
-    }
-
-    /// Stage the changes made to `path`.
-    fn add<'r>(repo: &'r git2::Repository, path: &Path) -> git2::Tree<'r> {
-        let mut index = repo.index().unwrap();
-        index.add_path(&path).unwrap();
-        index.write().unwrap();
-
-        let tree_id = index.write_tree_to(&repo).unwrap();
-        repo.find_tree(tree_id).unwrap()
-    }
-
-    /// Prepare an empty repo, and stage some changes.
-    fn prepare_and_stage() -> Context {
-        let (ctx, file_path) = prepare_repo();
-
-        // add some lines to our file
-        let path = ctx.join(&file_path);
-        let contents = std::fs::read_to_string(&path).unwrap();
-        let modifications = format!("new_line1\n{contents}\nnew_line2");
-        std::fs::write(&path, &modifications).unwrap();
-
-        // stage it
-        add(&ctx.repo, &file_path);
-
-        ctx
-    }
+    mod repo_utils;
 
     #[test]
     fn multiple_fixups_per_commit() {
-        let ctx = prepare_and_stage();
+        let ctx = repo_utils::prepare_and_stage();
 
         // run 'git-absorb'
         let drain = slog::Discard;
         let logger = slog::Logger::root(drain, o!());
-        let config = Config {
-            dry_run: false,
-            force: false,
-            base: None,
-            and_rebase: false,
-            whole_file: false,
-            one_fixup_per_commit: false,
-            logger: &logger,
-        };
-        run_with_repo(&config, &ctx.repo).unwrap();
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
 
         let mut revwalk = ctx.repo.revwalk().unwrap();
         revwalk.push_head().unwrap();
@@ -590,21 +539,16 @@ lines
 
     #[test]
     fn one_fixup_per_commit() {
-        let ctx = prepare_and_stage();
+        let ctx = repo_utils::prepare_and_stage();
 
         // run 'git-absorb'
         let drain = slog::Discard;
         let logger = slog::Logger::root(drain, o!());
         let config = Config {
-            dry_run: false,
-            force: false,
-            base: None,
-            and_rebase: false,
-            whole_file: false,
             one_fixup_per_commit: true,
-            logger: &logger,
+            ..DEFAULT_CONFIG
         };
-        run_with_repo(&config, &ctx.repo).unwrap();
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
 
         let mut revwalk = ctx.repo.revwalk().unwrap();
         revwalk.push_head().unwrap();
@@ -613,7 +557,295 @@ lines
         assert!(nothing_left_in_index(&ctx.repo).unwrap());
     }
 
-    fn autostage_common(ctx: &Context, file_path: &PathBuf) -> (PathBuf, PathBuf) {
+    #[test]
+    fn foreign_author() {
+        let ctx = repo_utils::prepare_and_stage();
+
+        repo_utils::become_author(&ctx.repo, "nobody2", "nobody2@example.com");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 1);
+        let is_something_in_index = !nothing_left_in_index(&ctx.repo).unwrap();
+        assert!(is_something_in_index);
+    }
+
+    #[test]
+    fn foreign_author_with_force_author_flag() {
+        let ctx = repo_utils::prepare_and_stage();
+
+        repo_utils::become_author(&ctx.repo, "nobody2", "nobody2@example.com");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            force_author: true,
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 3);
+
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+    }
+
+    #[test]
+    fn foreign_author_with_force_author_config() {
+        let ctx = repo_utils::prepare_and_stage();
+
+        repo_utils::become_author(&ctx.repo, "nobody2", "nobody2@example.com");
+
+        repo_utils::set_config_flag(&ctx.repo, "absorb.forceAuthor");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 3);
+
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+    }
+
+    #[test]
+    fn detached_head() {
+        let ctx = repo_utils::prepare_and_stage();
+        repo_utils::detach_head(&ctx.repo);
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let result = run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo);
+        assert_eq!(
+            result.err().unwrap().to_string(),
+            "HEAD is not a branch, use --force-detach to override"
+        );
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 1);
+        let is_something_in_index = !nothing_left_in_index(&ctx.repo).unwrap();
+        assert!(is_something_in_index);
+    }
+
+    #[test]
+    fn detached_head_pointing_at_branch_with_force_detach_flag() {
+        let ctx = repo_utils::prepare_and_stage();
+        repo_utils::detach_head(&ctx.repo);
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            force_detach: true,
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+
+        assert_eq!(revwalk.count(), 1); // nothing was committed
+        let is_something_in_index = !nothing_left_in_index(&ctx.repo).unwrap();
+        assert!(is_something_in_index);
+    }
+
+    #[test]
+    fn detached_head_with_force_detach_flag() {
+        let ctx = repo_utils::prepare_and_stage();
+        repo_utils::detach_head(&ctx.repo);
+        repo_utils::delete_branch(&ctx.repo, "master");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            force_detach: true,
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+
+        assert_eq!(revwalk.count(), 3);
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+    }
+
+    #[test]
+    fn detached_head_with_force_detach_config() {
+        let ctx = repo_utils::prepare_and_stage();
+        repo_utils::detach_head(&ctx.repo);
+        repo_utils::delete_branch(&ctx.repo, "master");
+
+        repo_utils::set_config_flag(&ctx.repo, "absorb.forceDetach");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+
+        assert_eq!(revwalk.count(), 3);
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+    }
+
+    #[test]
+    fn and_rebase_flag() {
+        let ctx = repo_utils::prepare_and_stage();
+        repo_utils::set_config_option(&ctx.repo, "core.editor", "true");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            and_rebase: true,
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+
+        assert_eq!(revwalk.count(), 1);
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+    }
+
+    #[test]
+    fn and_rebase_flag_with_rebase_options() {
+        let ctx = repo_utils::prepare_and_stage();
+        repo_utils::set_config_option(&ctx.repo, "core.editor", "true");
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            and_rebase: true,
+            rebase_options: &vec!["--signoff"],
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 1);
+
+        let trailers = message_trailers_strs(
+            ctx.repo
+                .head()
+                .unwrap()
+                .peel_to_commit()
+                .unwrap()
+                .message()
+                .unwrap(),
+        )
+        .unwrap();
+        assert_eq!(
+            trailers
+                .iter()
+                .filter(|trailer| trailer.0 == "Signed-off-by")
+                .count(),
+            1
+        );
+
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+    }
+
+    #[test]
+    fn rebase_options_without_and_rebase_flag() {
+        let ctx = repo_utils::prepare_and_stage();
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            rebase_options: &vec!["--some-option"],
+            ..DEFAULT_CONFIG
+        };
+        let result = run_with_repo(&logger, &config, &ctx.repo);
+
+        assert_eq!(
+            result.err().unwrap().to_string(),
+            "REBASE_OPTIONS were specified without --and-rebase flag"
+        );
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 1);
+        let is_something_in_index = !nothing_left_in_index(&ctx.repo).unwrap();
+        assert!(is_something_in_index);
+    }
+
+    #[test]
+    fn dry_run_flag() {
+        let ctx = repo_utils::prepare_and_stage();
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            dry_run: true,
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 1);
+        let is_something_in_index = !nothing_left_in_index(&ctx.repo).unwrap();
+        assert!(is_something_in_index);
+    }
+
+    #[test]
+    fn dry_run_flag_with_and_rebase_flag() {
+        let (ctx, path) = repo_utils::prepare_repo();
+        repo_utils::set_config_option(&ctx.repo, "core.editor", "true");
+
+        // create a fixup commit that 'git rebase' will act on if called
+        let tree = repo_utils::stage_file_changes(&ctx, &path);
+        let signature = ctx.repo.signature().unwrap();
+        let head_commit = ctx.repo.head().unwrap().peel_to_commit().unwrap();
+        ctx.repo
+            .commit(
+                Some("HEAD"),
+                &signature,
+                &signature,
+                &format!("fixup! {}\n", head_commit.id()),
+                &tree,
+                &[&head_commit],
+            )
+            .unwrap();
+
+        // stage one more change so 'git-absorb' won't exit early
+        repo_utils::stage_file_changes(&ctx, &path);
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        let config = Config {
+            and_rebase: true,
+            dry_run: true,
+            ..DEFAULT_CONFIG
+        };
+        run_with_repo(&logger, &config, &ctx.repo).unwrap();
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+        assert_eq!(revwalk.count(), 2); // git rebase wasn't called so both commits persist
+        let is_something_in_index = !nothing_left_in_index(&ctx.repo).unwrap();
+        assert!(is_something_in_index);
+    }
+
+    fn autostage_common(ctx: &repo_utils::Context, file_path: &PathBuf) -> (PathBuf, PathBuf) {
         // 1 modification w/o staging
         let path = ctx.join(&file_path);
         let contents = std::fs::read_to_string(&path).unwrap();
@@ -629,7 +861,7 @@ lines
 
     #[test]
     fn autostage_if_index_was_empty() {
-        let (ctx, file_path) = prepare_repo();
+        let (ctx, file_path) = repo_utils::prepare_repo();
 
         // requires enabled config var
         ctx.repo
@@ -643,16 +875,7 @@ lines
         // run 'git-absorb'
         let drain = slog::Discard;
         let logger = slog::Logger::root(drain, o!());
-        let config = Config {
-            dry_run: false,
-            force: false,
-            base: None,
-            and_rebase: false,
-            whole_file: false,
-            one_fixup_per_commit: false,
-            logger: &logger,
-        };
-        run_with_repo(&config, &ctx.repo).unwrap();
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
 
         let mut revwalk = ctx.repo.revwalk().unwrap();
         revwalk.push_head().unwrap();
@@ -663,7 +886,7 @@ lines
 
     #[test]
     fn do_not_autostage_if_index_was_not_empty() {
-        let (ctx, file_path) = prepare_repo();
+        let (ctx, file_path) = repo_utils::prepare_repo();
 
         // enable config var
         ctx.repo
@@ -674,21 +897,12 @@ lines
 
         let (_, fp2) = autostage_common(&ctx, &file_path);
         // we stage the extra file - should stay in index
-        add(&ctx.repo, &fp2);
+        repo_utils::add(&ctx.repo, &fp2);
 
         // run 'git-absorb'
         let drain = slog::Discard;
         let logger = slog::Logger::root(drain, o!());
-        let config = Config {
-            dry_run: false,
-            force: false,
-            base: None,
-            and_rebase: false,
-            whole_file: false,
-            one_fixup_per_commit: false,
-            logger: &logger,
-        };
-        run_with_repo(&config, &ctx.repo).unwrap();
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
 
         let mut revwalk = ctx.repo.revwalk().unwrap();
         revwalk.push_head().unwrap();
@@ -699,7 +913,7 @@ lines
 
     #[test]
     fn do_not_autostage_if_not_enabled_by_config_var() {
-        let (ctx, file_path) = prepare_repo();
+        let (ctx, file_path) = repo_utils::prepare_repo();
 
         // disable config var
         ctx.repo
@@ -713,16 +927,7 @@ lines
         // run 'git-absorb'
         let drain = slog::Discard;
         let logger = slog::Logger::root(drain, o!());
-        let config = Config {
-            dry_run: false,
-            force: false,
-            base: None,
-            and_rebase: false,
-            whole_file: false,
-            one_fixup_per_commit: false,
-            logger: &logger,
-        };
-        run_with_repo(&config, &ctx.repo).unwrap();
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
 
         let mut revwalk = ctx.repo.revwalk().unwrap();
         revwalk.push_head().unwrap();
@@ -730,4 +935,43 @@ lines
 
         assert!(nothing_left_in_index(&ctx.repo).unwrap());
     }
+
+    #[test]
+    fn fixup_message_always_commit_sha_if_configured() {
+        let ctx = repo_utils::prepare_and_stage();
+
+        ctx.repo
+            .config()
+            .unwrap()
+            .set_bool(config::FIXUP_TARGET_ALWAYS_SHA_CONFIG_NAME, true)
+            .unwrap();
+
+        // run 'git-absorb'
+        let drain = slog::Discard;
+        let logger = slog::Logger::root(drain, o!());
+        run_with_repo(&logger, &DEFAULT_CONFIG, &ctx.repo).unwrap();
+        assert!(nothing_left_in_index(&ctx.repo).unwrap());
+
+        let mut revwalk = ctx.repo.revwalk().unwrap();
+        revwalk.push_head().unwrap();
+
+        let oids: Vec<git2::Oid> = revwalk.by_ref().collect::<Result<Vec<_>, _>>().unwrap();
+        assert_eq!(oids.len(), 3);
+
+        let commit = ctx.repo.find_commit(oids[0]).unwrap();
+        let actual_msg = commit.summary().unwrap();
+        let expected_msg = format!("fixup! {}", oids.last().unwrap());
+        assert_eq!(actual_msg, expected_msg);
+    }
+
+    const DEFAULT_CONFIG: Config = Config {
+        dry_run: false,
+        force_author: false,
+        force_detach: false,
+        base: None,
+        and_rebase: false,
+        rebase_options: &Vec::new(),
+        whole_file: false,
+        one_fixup_per_commit: false,
+    };
 }
