@@ -7,19 +7,22 @@ use crate::config;
 pub fn working_stack<'repo>(
     repo: &'repo git2::Repository,
     user_provided_base: Option<&str>,
-    force: bool,
+    force_author: bool,
+    force_detach: bool,
     logger: &slog::Logger,
 ) -> Result<Vec<git2::Commit<'repo>>> {
     let head = repo.head()?;
     debug!(logger, "head found"; "head" => head.name());
 
     if !head.is_branch() {
-        if !force {
-            return Err(anyhow!("HEAD is not a branch, use --force to override"));
+        if !force_detach {
+            return Err(anyhow!(
+                "HEAD is not a branch, use --force-detach to override"
+            ));
         } else {
             warn!(
                 logger,
-                "HEAD is not a branch, but --force used to continue."
+                "HEAD is not a branch, but --force-detach used to continue."
             );
         }
     }
@@ -32,7 +35,9 @@ pub fn working_stack<'repo>(
 
     let base_commit = match user_provided_base {
         // https://github.com/rust-lang/rfcs/issues/1815
-        Some(commitish) => Some(repo.find_commit(repo.revparse_single(commitish)?.id())?),
+        // user_provided_base isn't guaranteed to be a commit hash, so peel until a
+        // commit is found.
+        Some(commitish) => Some(repo.revparse_single(commitish)?.peel_to_commit()?),
         None => None,
     };
 
@@ -67,11 +72,11 @@ pub fn working_stack<'repo>(
             break;
         }
         if let Ok(ref sig) = sig {
-            if !force
+            if !force_author
                 && (commit.author().name_bytes() != sig.name_bytes()
                     || commit.author().email_bytes() != sig.email_bytes())
             {
-                warn!(logger, "Will not fix up past commits not authored by you, use --force to override";
+                warn!(logger, "Will not fix up past commits not authored by you, use --force-author to override";
                       "commit" => commit.id().to_string());
                 break;
             }
@@ -192,7 +197,7 @@ mod tests {
 
         assert_stack_matches_chain(
             1,
-            &working_stack(&repo, None, false, &empty_slog()).unwrap(),
+            &working_stack(&repo, None, false, false, &empty_slog()).unwrap(),
             &commits,
         );
     }
@@ -209,6 +214,7 @@ mod tests {
                 &repo,
                 Some(&commits[0].id().to_string()),
                 false,
+                false,
                 &empty_slog(),
             )
             .unwrap(),
@@ -230,7 +236,7 @@ mod tests {
 
         assert_stack_matches_chain(
             config::MAX_STACK + 1,
-            &working_stack(&repo, None, false, &empty_slog()).unwrap(),
+            &working_stack(&repo, None, false, false, &empty_slog()).unwrap(),
             &commits,
         );
     }
@@ -247,7 +253,7 @@ mod tests {
 
         assert_stack_matches_chain(
             2,
-            &working_stack(&repo, None, false, &empty_slog()).unwrap(),
+            &working_stack(&repo, None, false, false, &empty_slog()).unwrap(),
             &new_commits,
         );
     }
@@ -265,7 +271,7 @@ mod tests {
 
         assert_stack_matches_chain(
             2,
-            &working_stack(&repo, None, false, &empty_slog()).unwrap(),
+            &working_stack(&repo, None, false, false, &empty_slog()).unwrap(),
             &commits,
         );
     }
